"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Redis = require("ioredis");
const Observable_1 = require("rxjs/Observable");
class PubsubManager {
    constructor(options) {
        this.options = Object.assign({}, this.getDefaultOptions(), options);
        this.redisClient = new Redis(this.options);
        this.topicMaps = new Map();
        this.serverEventsToObservables = new Map();
    }
    publish(topic, message) {
        return this.redisClient.publish(topic, message);
    }
    consume(topic) {
        return this.topicMaps.has(topic)
            ? this.topicMaps.get(topic)
            : this.createNewTopicObservable(topic);
    }
    getServerEventStream(eventName) {
        return this.serverEventsToObservables.has(eventName)
            ? this.serverEventsToObservables.get(eventName)
            : this.createNewEventObservable(eventName);
    }
    unsubscribe(topic) {
        this.redisClient.unsubscribe(topic);
        this.topicMaps.delete(topic);
    }
    createNewTopicObservable(topic) {
        let shouldAddToSubscribedTopics = true;
        let newObservable = Observable_1.Observable.create(observer => {
            this.redisClient.subscribe(topic, (err, numberOfChannels) => {
                if (err) {
                    console.log('Got error- ', err);
                    shouldAddToSubscribedTopics = false;
                    observer.error(err);
                }
                else {
                    console.log('connected to new channels, number is- ', numberOfChannels);
                }
            });
            this.redisClient.on('message', (channel, message) => {
                if (channel === topic) {
                    observer.next(message);
                }
            });
        });
        if (shouldAddToSubscribedTopics) {
            this.topicMaps.set(topic, newObservable);
        }
        return newObservable;
    }
    createNewEventObservable(event) {
        let eventObservable = Observable_1.Observable.create(observer => {
            this.redisClient.on(event, () => {
                observer.next();
            });
        });
        this.serverEventsToObservables.set(event, eventObservable);
        return eventObservable;
    }
    getDefaultOptions() {
        return {
            host: 'localhost',
            port: 6379,
            retryStrategy: times => {
                let delay = Math.min(100 + times * 2, 2000);
                return delay;
            },
            reconnectOnError: () => true
        };
    }
}
exports.PubsubManager = PubsubManager;
//# sourceMappingURL=redis-messaging-manager.js.map